from datetime import timedelta

from factory import Sequence, post_generation
from factory.django import DjangoModelFactory

from django.contrib.auth.models import Permission, User
from django.db.models import QuerySet
from django.utils.timezone import now
from program.models import (
    Category,
    FundingCategory,
    Host,
    Image,
    Language,
    License,
    LinkType,
    MusicFocus,
    Note,
    RRule,
    Schedule,
    Show,
    TimeSlot,
    Topic,
    Type,
)


class CommonUserFactory(DjangoModelFactory):
    class Meta:
        model = User

    password = "password"
    username = Sequence(lambda n: "common_%d" % n)


class UserWithPermissionsFactory(DjangoModelFactory):
    class Meta:
        model = User

    username = Sequence(lambda n: "user_with_perm%d" % n)

    @post_generation
    def user_permissions(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            if isinstance(extracted, Permission):
                self.user_permissions.add(extracted)

            if isinstance(extracted, QuerySet):
                for permission in extracted:
                    self.user_permissions.add(permission)


class HostFactory(DjangoModelFactory):
    class Meta:
        model = Host

    name = Sequence(lambda n: "host %d" % n)


class ImageFactory(DjangoModelFactory):
    class Meta:
        model = Image


class FundingCategoryFactory(DjangoModelFactory):
    class Meta:
        model = FundingCategory

    name = Sequence(lambda n: "funding category %d" % n)
    slug = Sequence(lambda n: "fc_%d" % n)


class TypeFactory(DjangoModelFactory):
    class Meta:
        model = Type

    name = Sequence(lambda n: "type %d" % n)
    slug = Sequence(lambda n: "t_%d" % n)


class ShowFactory(DjangoModelFactory):
    class Meta:
        model = Show

    name = Sequence(lambda n: "show %d" % n)
    slug = Sequence(lambda n: "%s_d" % n)


class RRuleFactory(DjangoModelFactory):
    class Meta:
        model = RRule


class ScheduleFactory(DjangoModelFactory):
    class Meta:
        model = Schedule

    end_time = "03:00:00"
    first_date = "2023-01-01"
    start_time = "02:00:00"


class TimeslotFactory(DjangoModelFactory):
    class Meta:
        model = TimeSlot

    end = now() + timedelta(hours=1)
    start = now()


class LicenseFactory(DjangoModelFactory):
    class Meta:
        model = License

    identifier = "pd"
    name = "Public Domain"


class NoteFactory(DjangoModelFactory):
    class Meta:
        model = Note


class CategoryFactory(DjangoModelFactory):
    class Meta:
        model = Category


class LanguageFactory(DjangoModelFactory):
    class Meta:
        model = Language


class LinkTypeFactory(DjangoModelFactory):
    class Meta:
        model = LinkType


class MusicFocusFactory(DjangoModelFactory):
    class Meta:
        model = MusicFocus


class TopicFactory(DjangoModelFactory):
    class Meta:
        model = Topic


class OwnerFactory(DjangoModelFactory):
    class Meta:
        model = User
